@@ -32,14 +32,12 @@ class GraphRepr(BaseRepr):
         file_or_url (`str`):
             The file or remote url that stores the information of this representation.
     """
-
-    def __init__(self, name: str, graph_type: str,
-                 op_reprs: Dict[str, OperatorRepr],
-                 df_reprs: Dict[str, DataFrameRepr]):
+    def __init__(self, name: str, graph_type: str, op_reprs: Dict[str, OperatorRepr], df_reprs: Dict[str, DataFrameRepr], ir: str=None):
         super().__init__(name)
         self._graph_type = graph_type
         self._operators = op_reprs
         self._dataframes = df_reprs
+        self._ir = ir
 
     @property
     def graph_type(self):
@@ -53,6 +51,10 @@ class GraphRepr(BaseRepr):
     def dataframes(self) -> Dict[str, DataFrameRepr]:
         return self._dataframes
 
+    @property
+    def ir(self) -> str:
+        return self._ir
+
     @staticmethod
     def dfs(cur: str, adj: Dict[str, List[str]], flag: Dict[str, int], cur_list: List[str]) -> Tuple[bool, List[str]]:
         """
@@ -192,7 +194,7 @@ class GraphRepr(BaseRepr):
             raise ValueError('file or src is not a valid YAML file to describe a DAG in Towhee.')
         dataframes = dict((df_info['name'], DataFrameRepr.from_dict(df_info)) for df_info in info['dataframes'])
         operators = dict((op_info['name'], OperatorRepr.from_dict(op_info)) for op_info in info['operators'])
-        return GraphRepr(info['name'], info.get('type', ''), operators, dataframes)
+        return GraphRepr(info['name'], info.get('type', ''), operators, dataframes, info.get('ir', None))
 
     @staticmethod
     def from_yaml(src: str):
@@ -234,7 +236,20 @@ class GraphRepr(BaseRepr):
                 The GraphRepr object.
         """
         info = BaseRepr.load_src(src)
-        return GraphRepr.from_dict(info)
+        g_repr = GraphRepr.from_dict(info)
+
+        iso_df = g_repr.get_isolated_df()
+        iso_op = g_repr.get_isolated_op()
+        loop = g_repr.get_loop()
+
+        err_msg = (
+            f'The YAML contains isolated dataframe {iso_df}. ' if bool(iso_df) else '' +
+            f'The YAML contains isolated dataframe {iso_op}. ' if bool(iso_op) else '' + f'The YAML contains loop {loop}' if bool(loop) else ''
+        )
+        if bool(iso_df) or bool(iso_op) or bool(loop):
+            raise ValueError(err_msg)
+
+        return g_repr
 
     def to_yaml(self) -> str:
         """Export a YAML file describing this graph.
