@@ -14,13 +14,21 @@
 
 
 from pathlib import Path
+from .operator_registry import OperatorRegistry # pylint: disable=import-outside-toplevel
 
-_DEFAULT_LOCAL_CACHE_ROOT = Path.home() / '.towhee'
-LOCAL_PIPELINE_CACHE = _DEFAULT_LOCAL_CACHE_ROOT / 'pipelines'
-LOCAL_OPERATOR_CACHE = _DEFAULT_LOCAL_CACHE_ROOT / 'operators'
+
+register = OperatorRegistry.register
+resolve = OperatorRegistry.resolve
+
+
+DEFAULT_LOCAL_CACHE_ROOT = Path.home() / '.towhee'
+LOCAL_PIPELINE_CACHE = DEFAULT_LOCAL_CACHE_ROOT / 'pipelines'
+LOCAL_OPERATOR_CACHE = DEFAULT_LOCAL_CACHE_ROOT / 'operators'
 
 MOCK_PIPELINES = str(Path(__file__).parent.parent /
                      'tests/test_util/resnet50_embedding.yaml')
 
 MOCK_OPS = str(Path(__file__).parent.parent / 'tests/mock_operators')
 MOCK_PIPES = str(Path(__file__).parent.parent / 'tests/mock_pipelines')
+
+
