@@ -14,13 +14,14 @@
 
 from typing import Dict
 from enum import Enum, auto
+from collections import defaultdict
 
 from towhee.dag.operator_repr import OperatorRepr
 from towhee.engine.operator_runner.runner_base import RunnerStatus
 from towhee.dataframe import DataFrame
 from towhee.engine.operator_io import create_reader, create_writer
 from towhee.engine.operator_runner import create_runner
-from towhee.engine.thread_pool_task_executor import ThreadPoolExecutor
+from towhee.engine.thread_pool_task_executor import ThreadPoolTaskExecutor
 
 
 class OpStatus(Enum):
@@ -38,7 +39,6 @@ class OperatorContext:
     The abstraction of OperatorContext hides the complexity of Dataframe management,
     input iteration, and data dependency between Operators. It offers a Task-based
     scheduling context.
-
     Args:
         op_repr: (OperatorRepr)
             The operator representation
@@ -46,29 +46,37 @@ class OperatorContext:
             All the `DataFrames` in `GraphContext`
     """
 
-    def __init__(
-        self,
-        op_repr: OperatorRepr,
-        dataframes: Dict[str, DataFrame]
-    ):
+    def __init__(self, op_repr: OperatorRepr, dataframes: Dict[str, DataFrame]):
         self._repr = op_repr
-
-        inputs = list({dataframes[input['df']] for input in op_repr.inputs})
-        iter_type = op_repr.iter_info['type']
-        inputs_index = dict((item['name'], item['col'])
-                            for item in op_repr.inputs)
-        self.inputs = inputs
-        self._reader = create_reader(inputs, iter_type, inputs_index)
-
-        outputs = list({dataframes[output['df']]
-                       for output in op_repr.outputs})
-
-        self._writer = create_writer(iter_type, outputs)
+        self._readers = OperatorContext._create_reader(op_repr, dataframes)
+        self._writer = OperatorContext._create_writer(op_repr, dataframes)
         self._op_runners = []
-
         self._op_status = OpStatus.NOT_RUNNING
         self._err_msg = None
 
+    @staticmethod
+    def _create_reader(op_repr, dataframes):
+        inputs_index = defaultdict(dict)
+        for item in op_repr.inputs:
+            inputs_index[item['df']][item['name']] = item['col']
+        iter_type = op_repr.iter_info['type']
+        iter_params = op_repr.iter_info.get('params')
+
+        inputs = dict((item['df'], dataframes[item['df']]) for item in op_repr.inputs)
+        readers = []
+        for df_name, indexs in inputs_index.items():
+            readers.append(create_reader(inputs[df_name], iter_type, indexs, iter_params))
+        return readers
+
+    @staticmethod
+    def _create_writer(op_repr, dataframes):
+        '''
+        Normally one op one output dataframe.
+        '''
+        outputs = list({dataframes[output['df']] for output in op_repr.outputs})
+        iter_type = op_repr.iter_info['type']
+        return create_writer(iter_type, outputs)
+
     @property
     def name(self):
         return self._repr.name
@@ -100,20 +108,45 @@ class OperatorContext:
             self._op_status = OpStatus.FINISHED
         return self._op_status
 
-    def start(self, executor: ThreadPoolExecutor, count: int = 1) -> None:
+    def start(self, executor: ThreadPoolTaskExecutor, count: int = 1) -> None:
         if self._op_status != OpStatus.NOT_RUNNING:
             raise RuntimeError('OperatorContext can only be started once')
 
         self._op_status = OpStatus.RUNNING
 
-        for i in range(count):
-            self._op_runners.append(
-                create_runner(self._repr.iter_info['type'], self._repr.name, i, self._repr.name,
-                              self._repr.function, self._repr.init_args, self._reader, self._writer)
-            )
+        try:
+            for i in range(count):
+                self._op_runners.append(
+                    create_runner(
+                        self._repr.iter_info['type'],
+                        self._repr.name,
+                        i,
+                        self._repr.name,
+                        self._repr.tag,
+                        self._repr.function,
+                        self._repr.init_args,
+                        self._readers,
+                        self._writer,
+                    )
+                )
+        except AttributeError as e:
+            self._err_msg = str(e)
+            self._op_status = OpStatus.FAILED
+            return
+
         for runner in self._op_runners:
             executor.push_task(runner)
 
+    def slow_down(self, time_sec: int):
+        if self.status == OpStatus.RUNNING:
+            for runner in self._op_runners:
+                runner.slow_down(time_sec)
+
+    def speed_up(self):
+        if self.status == OpStatus.RUNNING:
+            for runner in self._op_runners:
+                runner.speed_up()
+
     def stop(self):
         if self.status != OpStatus.RUNNING:
             raise RuntimeError('Op ctx is already not running.')
@@ -122,7 +155,7 @@ class OperatorContext:
             runner.set_stop()
 
     def join(self):
-        # Waits all runner finished.
+        # Wait all runners finished.
         for runner in self._op_runners:
             runner.join()
         self._writer.close()
