@@ -11,15 +11,17 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import time
 from typing import Dict, Tuple
 from abc import ABC
 from enum import Enum, auto
 from threading import Event
 import traceback
 
+from towhee.types._frame import FRAME, _Frame
 from towhee.engine.status import Status
 from towhee.utils.log import engine_log
+from towhee.hparam import param_scope
 
 
 class RunnerStatus(Enum):
@@ -38,11 +40,10 @@ class _OpInfo:
     Operator hub info.
     """
 
-    def __init__(self, op_name: str, hub_op_id: str,
-                 op_args: Dict[str, any]) -> None:
+    def __init__(self, op_name: str, hub_op_id: str, op_args: Dict[str, any], tag: str) -> None:
         self._op_name = op_name
         self._hub_op_id = hub_op_id
-        self._msg = None
+        self._tag = tag
         self._op_args = op_args
 
     @property
@@ -57,6 +58,10 @@ class _OpInfo:
     def op_args(self):
         return self._op_args
 
+    @property
+    def tag(self):
+        return self._tag
+
 
 class RunnerBase(ABC):
     """
@@ -65,16 +70,25 @@ class RunnerBase(ABC):
     The base class provides some function to control status.
     """
 
-    def __init__(self, name: str, index: int,
-                 op_name: str, hub_op_id: str,
-                 op_args: Dict[str, any],
-                 readers=None, writer=None) -> None:
+    def __init__(
+        self,
+        name: str,
+        index: int,
+        op_name: str,
+        tag: str,
+        hub_op_id: str,
+        op_args: Dict[str, any],
+        readers=None,
+        writer=None,
+    ) -> None:
         self._name = name
         self._index = index
         self._status = RunnerStatus.IDLE
-        self._op_info = _OpInfo(op_name, hub_op_id, op_args)
+        self._msg = None
+        self._op_info = _OpInfo(op_name, hub_op_id, op_args, tag)
 
         self._readers = readers
+        self._sleep_time = 0
 
         # only concat can have multiple readers
         if self._readers:
@@ -97,6 +111,10 @@ class RunnerBase(ABC):
     def hub_op_id(self):
         return self._op_info.hub_op_id
 
+    @property
+    def tag(self):
+        return self._op_info.tag
+
     @property
     def op_args(self):
         return self._op_info.op_args
@@ -130,7 +148,6 @@ class RunnerBase(ABC):
 
     def _set_failed(self, msg: str) -> None:
         error_info = '{} runs failed, error msg: {}'.format(str(self), msg)
-        engine_log.error(error_info)
         self._msg = error_info
         self._set_end_status(RunnerStatus.FAILED)
 
@@ -157,7 +174,9 @@ class RunnerBase(ABC):
 
     def _call_op(self, inputs) -> Status:
         try:
-            outputs = self._op(**inputs)
+            with param_scope() as hp:
+                hp().towhee.need_schema = True
+                outputs = self._op(**inputs)
             return Status.ok_status(outputs)
         except Exception as e:  # pylint: disable=broad-except
             err = '{}, {}'.format(str(e), traceback.format_exc())
@@ -165,12 +184,21 @@ class RunnerBase(ABC):
 
     def _get_inputs(self) -> Tuple[bool, Dict[str, any]]:
         try:
-            data = self._reader.read()
+            data, self._row_data = self._reader.read()
+            assert isinstance(self._row_data[-1], _Frame)
+            self._frame = self._row_data[-1]
+            self._frame.prev_id = self._frame.row_id
             return False, data
         except StopIteration:
             return True, None
 
     def _set_outputs(self, output: any):
+        if hasattr(output, '_asdict'):
+            output = output._asdict()
+
+        if isinstance(output, dict):
+            output.update({FRAME: self._frame})
+
         self._writer.write(output)
 
     def process_step(self) -> bool:
@@ -192,6 +220,16 @@ class RunnerBase(ABC):
             self._set_failed(st.msg)
             return True
 
+    def slow_down(self, time_sec: float):
+        self._sleep_time = time_sec
+
+    def speed_up(self):
+        self._sleep_time = 0
+
+    def sleep(self):
+        if self._sleep_time > 0:
+            time.sleep(self._sleep_time)
+
     def join(self):
         self._end_event.wait()
 
@@ -210,3 +248,4 @@ class RunnerBase(ABC):
             else:
                 self._set_finished()
                 break
+            self.sleep()
