@@ -15,12 +15,12 @@
 from typing import Tuple, Union, Dict, List
 import traceback
 
-from towhee.engine.operator_runner.map_runner import MapRunner
+from towhee.engine.operator_runner.runner_base import RunnerBase
 from towhee.engine.status import Status
-from towhee.utils.log import engine_log
+from towhee.errors import OpIOTypeError
 
 
-class WindowRunner(MapRunner):
+class WindowRunner(RunnerBase):
     """
     WindowRunner, multiple inputs and multiple outputs.
 
@@ -28,12 +28,19 @@ class WindowRunner(MapRunner):
     If run an op error, we should pass error info by an error handler.
     """
 
+    def _get_inputs(self) -> Tuple[bool, Dict[str, any]]:
+        try:
+            data = self._reader.read()
+            return False, data
+        except StopIteration:
+            return True, None
+
     def _set_outputs(self, output: List[any]):
         if not isinstance(output, list):
-            raise RuntimeError("Window operator's output must be a list, not a {}".format(type(output)))
+            raise OpIOTypeError("Window operator's output must be a list, not a {}".format(type(output)))
 
         for data in output:
-            self._writer.write(data)
+            self._writer.write(data._asdict())
 
     def _call_op(self, inputs: List[Dict]) -> Tuple[bool, Union[str, any]]:
         try:
@@ -41,5 +48,4 @@ class WindowRunner(MapRunner):
             return Status.ok_status(outputs)
         except Exception as e:  # pylint: disable=broad-except
             err = "{}, {}".format(e, traceback.format_exc())
-            engine_log.error(err)
             return Status.err_status(err)
