@@ -14,6 +14,7 @@
 
 from towhee.dag.graph_repr import GraphRepr
 from towhee.dataframe import DataFrame
+from towhee.errors import NoSchedulerError, EmptyInputError
 from towhee.engine.graph_context import GraphContext
 from towhee.engine.task_scheduler import TaskScheduler
 
@@ -36,16 +37,25 @@ class Pipeline:
         self._parallelism = parallelism
         self.on_graph_finish_handlers = []
         self._scheduler = None
+        self._graph_count = 0
 
         if isinstance(graph_repr, str):
             self._graph_repr = GraphRepr.from_yaml(graph_repr)
         else:
             self._graph_repr = graph_repr
 
+    @property
+    def pipeline_type(self) -> int:
+        return self._graph_repr.graph_type
+
     @property
     def parallelism(self) -> int:
         return self._parallelism
 
+    @property
+    def graph_repr(self) -> GraphRepr:
+        return self._graph_repr
+
     @parallelism.setter
     def parallelism(self, val):
         if isinstance(val, int) and val > 0:
@@ -56,6 +66,12 @@ class Pipeline:
     def register(self, scheduler: TaskScheduler):
         self._scheduler = scheduler
 
+    def __repr__(self) -> str:
+        if self._graph_repr._ir is not None:
+            return self._graph_repr._ir
+        else:
+            return object.__repr__(self)
+
     def __call__(self, inputs: DataFrame) -> DataFrame:
         """
         Process an input `DataFrame`. This function instantiates an output `DataFrame`;
@@ -77,15 +93,15 @@ class Pipeline:
                 Output `DataFrame` with ordering matching the input `DataFrame`.
         """
         if not self._scheduler:
-            raise AttributeError('Pipeline not registered to a Scheduler.')
+            raise NoSchedulerError('Pipeline not registered to a Scheduler.')
 
         assert self._parallelism == 1
-        graph_ctx = GraphContext(0, self._graph_repr)
+        graph_ctx = GraphContext(self._graph_count, self._graph_repr)
+        self._graph_count += 1
         self._scheduler.register(graph_ctx)
         input_data = inputs.get(0, 1)
         if input_data is None:
-            raise RuntimeError('Input data is empty')
-
+            raise EmptyInputError('Input data is empty')
         graph_ctx(input_data[0])
         graph_ctx.join()
         return graph_ctx.result()
