@@ -13,13 +13,21 @@
 # limitations under the License.
 
 import threading
+import os
 from pathlib import Path
 from typing import Union, List
 from shutil import copy2, copytree, rmtree
+try:
+    import importlib.resources as importlib_resources
+except ModuleNotFoundError:
+    import importlib_resources
 
-from towhee.engine.singleton import singleton
+from towhee.utils.singleton import singleton
 from towhee.engine import DEFAULT_LOCAL_CACHE_ROOT
-from towhee.hub.hub_tools import download_repo
+from towhee.utils.log import engine_log
+from towhee.hub.pipeline_manager import PipelineManager
+from towhee.hub.operator_manager import OperatorManager
+from towhee.utils.git_utils import GitUtils
 
 
 @singleton
@@ -124,8 +132,7 @@ class FileManagerConfig():
                 try:
                     self._cache_paths.remove(Path(path))
                 except ValueError:
-                    # TODO: Log this instead of print.
-                    print(str(Path(path)) + ' not found.')
+                    engine_log.error('%s%s ', str(Path(path)), ' not found.')
 
             if len(self._cache_paths) <= 1:
                 self._cache_paths.append(default_path)
@@ -222,15 +229,17 @@ class FileManager():
         self._cache_locals()
 
     # Move to a utils location?
-    def _cache_name(self, name: str, author: str, branch: str):
-        # If pointing directly to pipeline file.
-        if name.endswith('.yaml'):
-            return name[:-5] + '&' + author + '$' + branch
-        # If pointing directly to operator file.
-        elif name.endswith('.py'):
-            return name[:-3] + '&' + author + '$' + branch
+    def _cache_name(self, repo: str, author: str, tag: str):
+        try:
+            file_name, file_type = repo.split('.')
+        except ValueError:
+            return Path(author) / repo.replace('-', '_') / tag
+        # If pointing directly to pipeline or operator file.
+        if file_type in ['py', 'yaml']:
+            return Path(author) / file_name.replace('-', '_') / tag
+        # If not pointing to a pipeline or an operator file.
         else:
-            return name + '&' + author + '$' + branch
+            raise ValueError('Unsupported file type.')
 
     # TODO: filip-halt
     # Decide if to treat pipelines and operators the same. Should we allow single .py
@@ -238,7 +247,7 @@ class FileManager():
     def _cache_locals(self):
         with self._pipeline_lock and self._config.pipeline_lock:
             for pipe in self._config.cache_pipelines:
-                new_dir = self._cache_name(pipe['name'], author='local', branch='main')
+                new_dir = self._cache_name(pipe['name'], author='local', tag='main')
                 file = pipe['name']
                 cache_path = pipe['cache']
                 old_path = pipe['path']
@@ -250,7 +259,7 @@ class FileManager():
                 if new_path.is_file():
                     # TODO: filip-halt
                     # Error logging.
-                    print('Did not overwrite.')
+                    engine_log.info('Did not overwrite.')
                 else:
                     # TODO: filip-halt
                     # Figure out exception types.
@@ -259,7 +268,7 @@ class FileManager():
 
         with self._operator_lock and self._config.operator_lock:
             for op in self._config.cache_operators:
-                new_dir = self._cache_name(op['name'], author='local', branch='main')
+                new_dir = self._cache_name(op['name'], author='local', tag='main')
                 cache_path = op['cache']
                 old_path = op['path']
                 new_path = cache_path / 'operators' / new_dir
@@ -270,26 +279,28 @@ class FileManager():
                 if new_path.is_dir():
                     # TODO: filip-halt
                     # Error logging.
-                    print('Did not overwrite.')
+                    engine_log('Did not overwrite.')
                 else:
                     # TODO: Figure out exception types
                     copytree(str(old_path), str(new_path))
 
-    def get_pipeline(self, pipeline: str, branch: str = 'main', redownload: bool = False, install_reqs=True):
+    def get_builtin_pipeline(self, file_name: str) -> str:
+        with importlib_resources.path('towhee.hub.builtin.pipelines', '{}.yaml'.format(file_name)) as path:
+            return path
+
+    def get_pipeline(self, pipeline: str, tag: str, install_reqs: bool = True):
         """
         Obtain the path to the requested pipeline.
 
         This function will obtain the first reference to the pipeline from the cache locations.
         If no pipeline is found, this function will download it to the default cache location
-        from hub.towhee.io.
+        from the Towhee hub.
 
         Args:
             pipeline (`str`):
                 The pipeline in 'author/repo' format. Author will be 'local' if locally imported.
-            branch (`str`):
-                Which branch version to use of the pipeline. Branch will be 'main' if locally imported.
-            redownload (`bool`):
-                Whether to delete and redownload the pipeline files
+            tag (`str`):
+                Which tag to use of the pipeline. Will use 'main' if locally imported.
             install_reqs (`bool`):
                 Whether to download the python packages if a requirements.txt file is included in the repo.
 
@@ -308,14 +319,18 @@ class FileManager():
                 raise ValueError('''Incorrect pipeline format, should be '<author>/<pipeline_repo>'.''')
             author = pipeline_split[0]
             repo = pipeline_split[1]
+            file_name = repo.replace('-', '_')
 
-            pipeline_path = self._cache_name(repo, author, branch) + '/' + repo + '.yaml'
+            if author == 'builtin':
+                return self.get_builtin_pipeline(file_name)
+
+            # This path leads to 'author/repo/tag/file_name.yaml'
+            pipeline_path = self._cache_name(repo, author, tag) / (file_name + '.yaml')
 
-            file_path = self._config.default_cache / 'pipelines' / pipeline_path
             found_existing = False
-
-            for path in self._config.cache_paths:
-                path = path / 'pipelines' / pipeline_path
+            for path_iter in self._config.cache_paths:
+                # This path leads to 'cache/pipelines/author/repo/tag/file_name.yaml'
+                path = path_iter / 'pipelines' / pipeline_path
                 if path.is_file():
                     file_path = path
                     found_existing = True
@@ -323,36 +338,66 @@ class FileManager():
 
             if author == 'local':
                 if found_existing is False:
-                    # TODO: filip-halt
-                    # Error logging.
-                    print('Local file not found, has it been imported?')
-                    file_path = None
-            else:
-                if redownload:
-                    # TODO: filip-halt
-                    # Error logging.
-                    rmtree(file_path.parent)
+                    msg = f'Pipeline {pipeline} not find, has it been imported?'
+                    raise FileNotFoundError(msg)
+                return file_path
 
-                if file_path.is_file() is False:
-                    download_repo(author, repo, branch, str(file_path.parent), install_reqs=install_reqs)
+            if not found_existing:
+                file_path = self._config.default_cache / 'pipelines' / pipeline_path
+                repo_path = file_path.parent
+
+                # If yaml file missing but repo exists.
+                if repo_path.is_dir():
+                    engine_log.warning('Your local opreator repo %s exists, but not complete, \'.py\' file missing, please check on hub.', file_name)
+                    rmtree(repo_path)
+                    raise FileNotFoundError('Repo exists but \'.yaml\' file missing.')
+
+                # If user has git, clone the repo, otherwise try to download.
+                try:
+                    git = GitUtils(author, repo)
+                    git.clone(tag=tag, install_reqs=install_reqs, local_repo_path=repo_path)
+                # If user does not have git in the system, subprocess throws FileNotFound error.
+                except FileNotFoundError:
+                    engine_log.warning(
+                        '\'git\' not found, execute download instead of clone. ' \
+                        'If you want to check updates every time you run the pipeline, please install \'git\' and remove current local cache.'
+                    )
+                    pipeline_manager = PipelineManager(author=author, repo=repo)
+                    pipeline_manager.download(local_repo_path=file_path.parent, tag=tag, install_reqs=install_reqs)
+                    return file_path
+
+            # Check updates for repo.
+            elif tag == 'main':
+                repo_path = file_path.parent
+                cwd = Path.cwd()
+                os.chdir(repo_path)
+                try:
+                    git = GitUtils(author, repo)
+                    if 'Your branch is behind' in git.status():
+                        engine_log.warning('Your local pipeline %s is not up to date, updating to latest version...', file_name)
+                        git.pull()
+                except FileNotFoundError:
+                    engine_log.warning(
+                        '\'git\' not found, cannot update. ' \
+                        'If you want to check updates every time you run the pipeline, please install \'git\' and remove current local cache.'
+                    )
+                os.chdir(cwd)
 
         return file_path
 
-    def get_operator(self, operator: str, branch: str = 'main', redownload: bool = False, install_reqs=True):
+    def get_operator(self, operator: str, tag, install_reqs: bool = True):
         """
         Obtain the path to the requested operator.
 
         This function will obtain the first reference to the operator from the cache locations.
         If no opeartor is found, this function will download it to the default cache location
-        from hub.towhee.io.
+        from the Towhee hub.
 
         Args:
             operator (`str`):
                 The operator in 'author/repo' format. Author will be 'local' if locally imported.
-            branch (`str`):
-                Which branch version to use of the opeartor. Branch will be 'main' if locally imported.
-            redownload (`bool`):
-                Whether to delete and redownload the operator files
+            tag (`str`):
+                Which tag version to use of the opeartor. Will use 'main' if locally imported.
             install_reqs (`bool`):
                 Whether to download the python packages if a requirements.txt file is included in the repo.
 
@@ -371,14 +416,14 @@ class FileManager():
                 raise ValueError('''Incorrect operator format, should be '<author>/<operator_repo>'.''')
             author = operator_split[0]
             repo = operator_split[1]
+            file_name = repo.replace('-', '_')
+            # This path leads to 'author/repo/tag/file_name.yaml'
+            operator_path = self._cache_name(repo, author, tag) / (file_name + '.py')
 
-            operator_path = self._cache_name(repo, author, branch) + '/' + repo + '.py'
-
-            file_path = self._config.default_cache / 'operators' / operator_path
             found_existing = False
-
-            for path in self._config.cache_paths:
-                path = path / 'operators' / operator_path
+            for path_iter in self._config.cache_paths:
+                # This path leads to 'cache/operators/author/repo/tag/file_name.py'
+                path = path_iter / 'operators' / operator_path
                 if path.is_file():
                     file_path = path
                     found_existing = True
@@ -386,16 +431,48 @@ class FileManager():
 
             if author == 'local':
                 if found_existing is False:
-                    # TODO: filip-halt
-                    # Error logging.
-                    print('Local file not found, has it been imported?')
-                    file_path = None
-            else:
-                if redownload:
-                    # TODO: filip-halt
-                    # Error logging.
-                    rmtree(file_path.parent)
+                    msg = f'Operator {operator} not found, has it been imported?'
+                    raise FileNotFoundError(msg)
+                return file_path
+
+            if not found_existing:
+                file_path = self._config.default_cache / 'operators' / operator_path
+                repo_path = file_path.parent
+                # If py file missing but repo exists.
+                if repo_path.is_dir():
+                    engine_log.error('Your local opreator repo %s exists, but not complete, \'.py\' file missing, please check on hub.', file_name)
+                    rmtree(repo_path)
+                    raise FileNotFoundError('Repo exists but \'.py\' file missing.')
+
+                # If user has git, clone the repo, otherwise try to download.
+                try:
+                    git = GitUtils(author, repo)
+                    git.clone(tag=tag, install_reqs=install_reqs, local_repo_path=repo_path)
+                # If user does not have git in the system, subprocess throws FileNotFound error.
+                except FileNotFoundError:
+                    engine_log.warning(
+                        '\'git\' not found, execute download instead of clone. ' \
+                        'If you want to check updates every time you run the pipeline, please install \'git\' and remove current local cache.'
+                    )
+                    operator_manager = OperatorManager(author=author, repo=repo)
+                    operator_manager.download(local_repo_path=file_path.parent, tag=tag, install_reqs=install_reqs)
+                    return file_path
+
+            # Check updates fro repo.
+            elif tag == 'main':
+                repo_path = file_path.parent
+                cwd = Path.cwd()
+                os.chdir(repo_path)
+                try:
+                    git = GitUtils(author, repo)
+                    if 'Your branch is behind' in git.status():
+                        engine_log.warning('Your local operator %s is not up to date, updating to latest version...', file_name)
+                        git.pull()
+                except FileNotFoundError:
+                    engine_log.warning(
+                        '\'git\' not found, cannot update. ' \
+                        'If you want to check updates every time you run the pipeline, please install \'git\' and remove current local cache.'
+                    )
+                os.chdir(cwd)
 
-                if file_path.is_file() is False:
-                    download_repo(author, repo, branch, str(file_path.parent), install_reqs=install_reqs)
         return file_path
