@@ -12,10 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from towhee.operator.base import Operator, NNOperator, PyOperator, SharedType
 
-from towhee.operator.base import Operator, SharedType
-
-__all__ = [
-    "Operator",
-    "SharedType"
-]
+__all__ = ['Operator', 'NNOperator', 'PyOperator', 'SharedType']
